###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###

from pytonik import App


class Controllers(App):
    global data
    global model
    global params

    def getDate(self):
        return self.data

    def getModel(self):
        return self.model

    def getParams(self):
        return self.params

    def __init__(self, datag={}, datal={}):
        self.App = App
        self.datag = datag
        self.datal = datal
        self.params = self.App.getRouters()
        self.keys = dict()



# Author : BetaCodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by BetaCodings on 05/11/2019.
import os


class url:

    def __getattr__(self, item):
        return item

    def __call__(self, *args, **kwargs):

        return None

    def __init__(self, *args, **kwargs):
        if len(args) > 0 or len(kwargs) > 0:
            if all(args) is not False:
                self.ul = self.url(*args, **kwargs)
            else:
                self.ul = self.url(**kwargs)
        else:
            self.ul = self.url('')
        return None

    def __str__(self):

        return self.ul


    def url(self, path = ""):

        seturl = str("localhost:") + str(os.environ.get("SERVER_PORT", ''))
        http = os.environ.get("HTTPS")
        if http == 'on':
            url = str("https://") + os.environ.get("HTTP_HOST", seturl)
        else:
            url = str("http://") + os.environ.get("HTTP_HOST", seturl)

        DS = ""
        p = ""

        if path == "":
            DS = ""
        else:
            if path[:1] == "/":
                p = path[1:]
                DS = "/"
            else:
                p = path
                DS = "/"


        return url + DS + p



###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###



import cgi, os
from . import Router, Log
log_msg = Log.Log()

class Request:
 
    def __init__(self):
        self.attr = cgi.FieldStorage()
        self.type = os.environ
        self.Router = Router.Router()
        self.method = self.type.get('REQUEST_METHOD', '')


    def get(self, key=0, error=0):
        try:
            if 'GET' in self.type.get('REQUEST_METHOD'):

                if key != 0:
                    if (key in self.attr):
                        return self.attr.getvalue(key)
                    elif error == 1:
                        return self.attr
                    else:
                        return ""
                else:
                    return ""
            else:

                log_msg.info("advise use POST instead of GET")
                return False
        except Exception as err:
            log_msg.info(err)
            return  err

    def post(self, key=0, error=0):
        try:
            if 'POST' in self.type.get('REQUEST_METHOD'):
                if key != 0:
                    if (key in self.attr):
                        return self.attr.getvalue(key)
                    elif error == 1:
                        return self.attr
                    else:
                        return ""
                else:
                    return ""
            else:

                log_msg.info("advise use GET instead of POST")
                return False
        except Exception as err:
            log_msg.info(err)
            return err

    def file(self, key=0, error=0):
        try:
            if key != 0:
                if (key in self.attr):
                    self.attr.getvalue(key)
                    return self.attr[key]
                elif error == 1:
                    return self.attr
                else:
                    return ""
            else:
                return ""

        except Exception as err:
            log_msg.info(err)
            return err

    def all(self):
        if None is not self.attr.keys():
            return self.attr.keys()

    def params(self, key=0):
        try:
            if 'POST' in self.type.get('REQUEST_METHOD'):
                log_msg.info("advise use POST instead of PARAMS")
                return False
            else:
                para = self.Router.getParams()
                if para != "" or para is not None:
                    return para.get(key, '')
                else:
                    return self.get(key)

        except Exception as err:
            log_msg.info(err)





###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###


from pytonik.Request import Request
from pytonik.Router import Router
from pytonik.Editor import HTMLeditor
from pytonik.Config import Config
from pytonik.Log import Log
from pytonik import Lang
from pytonik import Version
from pytonik.Functions import url
import os, sys, cgi, cgitb, importlib, glob, inspect
from http import HTTPStatus

cgitb.enable()

log_msg = Log()

global host, u



if os.path.isdir(os.getcwd()+'/public'):
    host = os.getcwd() #os.path.dirname(os.getcwd())

else:
    host = os.path.dirname(os.getcwd())


DS = str("/")
u = url
controllerpath = host + DS + 'controller'
error_page_class= controllerpath + DS + "ErrorController" + ".py"
header_response_page = {
   '400': 'error/page400', #Bad Request
   '404': 'error/page404', #Not Found
   '405': 'error/page405', #Method Not Allowed

}


class App(Router):


    def __init__(self, routers="", routes="", getpath="", getrouters=""):

        self.routers = routers
        self.routes = routes
        self.getpath = getpath
        self.getrouters = getrouters
        self.getDB = ""
        self.Router = Router
        self.Request = Request
        self.methodprefix = ""
        self.actions = ""
        self.controllers = ""
        self.routersc = ""
        self.dbDriver = None
        self.Config = None
        self.url = ""
        self.params = ""
        self.key = ""
        self.vpathf = ""
        self.languages = ""
        self.Driver = ""

    def getRouters(self):

        return self.routers

    def getPath(self):
        return self.getpath


    def runs(self):

        self.getrouters = self.envrin('route')
        self.routersc = self.Router()
        self.methodprefix = self.routersc.getMethodPrefix()
        self.actions = self.routersc.getAction()
        self.controllers = self.routersc.getControllers()
        self.routers = self.routersc.getRoutes()
        self.languages = self.routersc.getLanguages()
        self.params = self.routersc.getParams()


        langs = Lang.Lang(self.languages)
        langs.loadLang()
        routesUri = []
        for k, getRouter in self.getrouters.items():

            if self.controllers == k:
                routesUri = getRouter.split('@')


        if len(routesUri) != 0:


            if 'controller' in routesUri[0].lower():

                controllersClass =  str(routesUri[0].lower()).replace('controller', '').capitalize()+ 'Controller'
            else:
                controllersClass = str(routesUri[0]).capitalize() + 'Controller'

            if ':' not in routesUri[1]:
                controllersMethods = str(routesUri[1])

            else:
                getMapPara = routesUri[1].split(':')

                controllersMethods = str(getMapPara[0])



        else:

            controllersClass = str(self.controllers[0].capitalize()) + str(self.controllers[1:]) + 'Controller'

            controllersMethods = str(self.actions)


        if controllersMethods != "":
            self.actions = controllersMethods


        controllers = controllerpath + DS + controllersClass + ".py"

        if os.path.isfile(controllers) == True:
            if __name__ == '__main__':
                spac = ""
            if os.path.isfile(controllers) == True:

                if sys.version_info.major <= 2:
                    self.strClass(controllerpath, controllersClass)
                else:
                    self.strClass3(controllerpath, controllersClass)

        else:

            self.errorP('405')



    def errorP(self, code="", replace = ""):
        getErrorP = self.envrin('error')

        pageCode = "page{code}".format(code=code)

        if getErrorP is not '':
            errorP = getErrorP

            if errorP.get(code, '') is not '':
               if '/' in  errorP.get(code, ''):
                    splitP = errorP.get(code, '').split('/')
                    controllerP = controllerpath + DS + str(splitP[0]).capitalize() + 'Controller'  + ".py"
               else:
                    controllerP = controllerpath + DS + str(errorP.get(code, '')).capitalize() + 'Controller'  + ".py"


               if os.path.isfile(controllerP) == True:
                   self.redirect(u.url().url('/'+str(errorP.get(code, ''))))

               else:

                   if  os.path.isfile(self.error_page_html(code)) == True:

                       self.redirect(u.url().url("/error/{code}".format(code=pageCode)))

                   else:
                       self.create_error_page(error_page_class)
                       self.redirect(u.url().url("/error/page{code}".format(code=code)))

            else:

                if  os.path.isfile(self.error_page_html(code)) == True:
                    self.redirect(u.url().url("/error/{code}".format(code=pageCode)))

                else:
                    self.create_error_page(error_page_class)
                    self.redirect(u.url().url("/error/{code}".format(code=pageCode)))

        else:
            if os.path.isfile(error_page_class) == True:
                self.redirect(u.url().url("/error/{code}".format(code=pageCode)))

            else:
                self.create_error_page(error_page_class)
                self.redirect(u.url().url("/error/{code}".format(code=pageCode)))





    def envrin(self,  key):
        env = self.env()
        self.Config = Config()
        self.Config.add(env)
        return self.Config.get(key, '')


    def DB(self):

        self.dbDriver = self.envrin('dbConnect')
        if self.dbDriver.get("driver", '') == "MYSQL":
            from pytonik.Driver.DB.MYSQL.MYSQL import MYSQL
            self.getDB = MYSQL(self.dbDriver)
            self.Driver = "MYSQL"
            return self.getDB

        if self.dbDriver.get("driver", '') == "SQLite":
            from pytonik.Driver.DB.SQLite.SQLite import SQLite
            self.getDB = SQLite(self.dbDriver)
            self.Driver = "SQLite"
            return self.getDB

        if self.dbDriver.get("driver", '') == "Oracle":
            from pytonik.Driver.DB.Oracle.Oracle import Oracle
            self.getDB = Oracle(self.dbDriver)
            self.Driver = "Oracle"
            return self.getDB

        if self.dbDriver.get("driver", '') == "pyPgSQL":
            from pytonik.Driver.DB.pyPgSQL.pyPgSQL import pyPgSQL
            self.getDB = pyPgSQL(self.dbDriver)
            self.Driver = "pyPgSQL"
            return self.getDB


    def strClass(self, p=None, c=None):

        try:
            sys.path.append(p)
            ms = str(self.actions)
            md = importlib.import_module(c)
            self.strMethod(md, ms)

        except Exception as err:
            log_msg.error(err)
            self.errorP('400')





    def strMethod(self, c=None, m=None):
        Request = self.Request()
        try:
            return getattr(c, m)(Request)
        except Exception as err:
            return getattr(c, m)()

    def strClass3(self, p=None, c=None):

        try:
            sys.path.append(p)
            ms = str(self.actions)
            importlib._RELOADING
            md = importlib.import_module(c, ms)
            self.strMethod(md, ms)

        except Exception as err:
            log_msg.error(err)
            self.errorP('400')


    def redirect(self, location='/'):
        print("Location: {location}".format(location=location))
        print()

    def referer(self, location='/'):
        print("Location: {location}".format(location=os.environ.get('HTTP_REFERER', location)))
        print()

    @staticmethod
    def header(p=0, type="text/html"):

        if Version.PYVERSION_MA <= 2 and Version.PYVERSION_MI <= 7:
            reload(sys)
            sys.setdefaultencoding('utf-8')
        elif Version.PYVERSION_MA == 3 and Version.PYVERSION_MI <= 3:
            import imp
            imp.reload()

        elif Version.PYVERSION_MA >= 3 and Version.PYVERSION_MI >= 4:
            import importlib
            importlib.reload(sys)
        print("Content-type: {type}\r\n\r\n".format(type=type))
        if p > 0:
            for x in range(p):
                print("")

    def views(self, pathf="", datag={}, datal={}):

        if pathf == "":
            pathf = self.getDefaultViewPath()

        pathfhtml = host + DS + 'views' + DS + pathf + ".html"

        if os.path.isfile(pathfhtml) == False:
            self.errorP('404')
        else:
            self.header()
            self.read_html(host + DS + 'views' + DS, pathf, datag)

        return False

    def read_html(self, template_dir, engine, context=[]):

        html_file_path = os.path.join(template_dir, "%s.html" % engine)


        try:
            with open(html_file_path) as html_file:
                html = html_file.read()

            print(HTMLeditor.Template(html).render(**context))

        except Exception as err:
            log_msg.error(err)

    def getDefaultViewPath(self):

        router = self.Router.getRoutes()  # Routers.getRoutes()

        if router == "":
            rout = ""
        controllerDirectory = self.Router.getControllers()

        templateName = str(self.Router.getMethodPrefix()) + str(self.Router.getAction()) + '.html'

        return templateName

    def error_page_html(self, code):
        return host+DS+'views'+DS+str(code)+".html"

    def create_error_page(self, error_page_class):


        try:
            python_writer_code = ""
            for code in header_response_page:
                if os.path.isfile(self.error_page_html(code)) == False:
                    f = open(self.error_page_html(code), 'a+')
                    f.write("<h1> {code}  Not Found</h1>".format(code=code))
                    f.close()
                    python_writer_code += """\ndef page{code} ():\n""".format(code=code)
                    python_writer_code += """\n    data = {'title': 'pytonik"""
                    python_writer_code += """ {code} """.format(code=code)
                    python_writer_code += """'}\n\n"""
                    python_writer_code += """    m.views('{code}', data)\n""".format(code=code)



            writer_code = """from pytonik import Web
                              \nm = Web.App()
                              \ndef index():
                              \n  m.header()
                              \n  print("do work here") \n
                            """
            code_line = str(writer_code)+str(python_writer_code)
            f = open(error_page_class, 'w')
            f.write(code_line)
            f.close()

        except Exception as err:
            log_msg.error(err)

    def loadmodule(self):

        path = [os.path.dirname(__file__)+ str("/") +str("Functions"), str(host) + str("/") + "model"]
        listpath = [path[0], path[1]]


        i = 0
        lclass = {}

        for pl in listpath:

            current_dir = os.path.join(pl)

            current_module_name = os.path.splitext(os.path.basename(current_dir))[0]

            for file in glob.glob(os.path.join(current_dir + "/*.py")):
                name = os.path.splitext(os.path.basename(file))[0]

                i = i + 1
                # Ignore __ files
                ++i
                if name.startswith("__init__"):
                    continue
                if name is not "__init__":
                    lclass0 = {name: name}
                    lclass.update(lclass0)

        if Version.PYVERSION_MA <= 2:
            item = lclass.iteritems()
        else:
            item = lclass.items()
        result = {}
        for key, value in item:
            if value not in result.values():
                result[key] = value

        return result



###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###



import sys, os, cgitb
from pytonik import Version, Config, Log
cgitb.enable()
url = os.environ.get('REQUEST_URI', os.environ.get('PATH_INFO'))
log_msg = Log.Log()


class Router:
    def __init__(self):
        urlstr = str(url)
        self.uri = urlstr.split('/')

        Conf = Config.Config()
        Conf.add(self.env())


        self.controllers = Conf.get('default_controllers')
        self.actions = Conf.get('default_actions')
        self.languages = Conf.get('default_languages')
        self.alllanguages = Conf.get('languages', '')
        self.routes = Conf.get('default_routes')
        self.methodprefix = ""
        self.params = ""



        #if "?" in self.uri:

            #uri_paths = urlstr.split("?")
            #path_array = uri_paths[0]
        #else:
            #uri_paths = urlstr.split('/')
            #path_array = uri_paths[0]

        uri_paths = urlstr.split('/')

        pathparts_array = uri_paths



        pathparts_paramarray = os.environ.get("QUERY_STRING", '')

        pathparts_paramarrayOut = dict()
        if pathparts_paramarray != '':
            pairs = pathparts_paramarray.split('&')

            pathparts_paramarray = pairs

            for i in pairs:

                name, value = i.split('=', 2)

                pathparts_paramarray = {name: value}

                if name in pathparts_paramarray:

                    pathparts_paramarray[name] = value

                else:
                    pathparts_paramarray[name] = [pathparts_paramarray[name], value]

                pathparts_paramarrayOut.setdefault(name, value)



        else:
            pathparts_paramarrayOut = ""

        path_parts = pathparts_array


        if len(path_parts):

            #print(routes.keys())

            if Version.PYVERSION_MA < 3:
                path_parts = filter(None, path_parts)
            else:
                path_parts = list(filter(None, path_parts))



            routes = Conf.get('route', '')

            if list(set(path_parts).intersection(routes.keys())):

                for s in path_parts:

                    if s in routes:
                        self.routes = s

                        if self.routes in routes:
                            self.methodprefix = routes[self.routes]
                        else:
                            self.methodprefix = ""

                        #path_parts.append(path_parts.pop(-1))


            languages = Conf.get('languages', '')


            if list(set(path_parts).intersection(languages.keys())):

                for s in path_parts:
                    if s in languages:
                        self.languages = s
                        path_parts.append(path_parts.pop(-1))


            controllers = Conf.get('default_controllers', '')
            if controllers:

                i = 0
                path_parts = list(filter(None, path_parts))

                for s in path_parts:

                    if s is not self.languages:
                        i += 1

                        if i == 1:
                            self.controllers = s
                            path_parts.append(path_parts.pop(-1))
                    ++i


            action = Conf.get('default_actions', '')
            if action:
                i = 0
                for s in path_parts:
                    if s is not self.controllers and s is not self.languages:
                        i += 1
                        if i == 1:
                            self.actions = s
                            path_parts.append(path_parts.pop(-1))
                        ++i


            from .Core import Helpers
            h = Helpers
            list_params = []

            if pathparts_paramarray == None or pathparts_paramarray == "" :
                if Version.PYVERSION_MA <= 2:
                    lroutes = routes.iteritems()
                else:
                    lroutes = routes.items()
                for k, getRouter in lroutes:

                    if self.controllers == k:

                        paraUri = getRouter.split('@')
                    else:
                        paraUri = []
                    if len(paraUri) > 0:
                        if ':' not in paraUri[1]:
                            getMapPara = []
                        else:
                            getMapPara = paraUri[1].split(':')


                        if self.controllers in routes:

                            if len(getMapPara[1:]) > 0:

                                new_para = path_parts[1:]

                                if len(new_para) > 0:
                                    param_m = []
                                    for i, para in enumerate(getMapPara[1:]):
                                        param_n = para
                                        v_para  = new_para[i]
                                        list_params.append(param_n)
                                        list_params.append(v_para)

                                self.params = Helpers.covert_list_dict(list_params)

                else:
                    for s in path_parts:

                        if s is not self.controllers and s is not self.actions and s is not self.languages:

                            list_params.append(s)

                            path_parts.append(path_parts.pop(-1))

                    self.params = Helpers.covert_list_dict(list_params)

            else:

                self.params = pathparts_paramarray
                path_parts.append(path_parts.pop(-1))


        return None

    def getUri(self):
        return self.uri

    def getControllers(self):

        return self.controllers

    def getAction(self):
        return self.actions

    def getParams(self):

        return self.params

    def getRoutes(self):

        return self.routes

    def getMethodPrefix(self):
        return self.methodprefix

    def getLanguages(self):


        return self.languages

    def env(self):

        import os

        if os.path.isdir(os.getcwd() + '/public'):
            host = os.getcwd()  # os.path.dirname(os.getcwd())

        else:
            host = os.path.dirname(os.getcwd())

        DS = str("/")

        envpath = host + DS + ".env"

        if os.path.isfile(envpath) == True:

            try:
                f = open(envpath, "r")
                return f.read()
            except Exception as err:
                    log_msg.error(err)
        else:
            log_msg.critical(".env file not found")
            return ".env file not found"



###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###


def App():
    from . import App
    return App.App()

def Config():
    from . import Config as Config
    return Config.Config()

def Helpers():
    from .Core import Helpers
    return Helpers

def Session():
    from .Session import Session
    return Session()

def Request():
    from .Request import Request
    return Request()

def Hash():
    from .Hash import Hash
    return Hash()

def Model():
    from .Model import Model
    M = Model()
    return M

def Load(m):
    M = Model()
    return M.load(m)

def Functions():
    from .Core import Functions
    return Functions

def Pagination():
    from pytonik.Functions.pagination import pagination
    return pagination

def Validation():
    from pytonik.Functions.validation import validation
    return validation

def curl():
    from pytonik.Functions.curl import curl
    return curl


def now():
    from pytonik.Functions.now import now
    return now


def extend():
    from pytonik.Functions.extend import extend
    return extend



def include():
    from pytonik.Functions.include import include
    return include


def File():
    from .Core import File

    return File

def SendEmail(from_send, to_recipient, message_subject= "", messege_content = ""):
    from .Core.SMTP import SMTP

    return SMTP().send(from_send, to_recipient, message_subject, messege_content)

def Logs():
    from .Log import Log

    return Log()


def url(path=""):
    import os
    seturl = str("localhost:") + str(os.environ.get("SERVER_PORT", ''))
    http = os.environ.get("HTTPS")
    if http == 'on':
        url = str("https://") + os.environ.get("HTTP_HOST", seturl)
    else:
        url = str("http://") + os.environ.get("HTTP_HOST", seturl)

    DS = ""
    p = ""

    if path == "":
        DS = ""
    else:
        if path[:1] == "/":
            p = path[1:]
            DS = "/"
        else:
            p = path
            DS = "/"

    return url + DS + p


###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###


import os, datetime, sys
from . import Version

if Version.PYVERSION_MA <= 3 and  Version.PYVERSION_MI < 7:
    import Cookie
else:
    from http import cookies


class Session:
    def __init__(self):
            self.result = ""


    def has(self, key=None):

        if key in self.settings:
            return True
        else:
            return False

    def set(self, key="", value="", duration = 10, url=os.environ.get("HTTP_HOST"), path="/"):
        expires = datetime.datetime.utcnow() + datetime.timedelta(minutes=duration)  # minutes in 30 days

        if Version.PYVERSION_MA >= 3:
            cooKeys = cookies.SimpleCookie(os.environ.get('HTTP_COOKIE'))
        else:
            cooKeys = Cookie.SimpleCookie(os.environ.get('HTTP_COOKIE'))

        cooKeys[str(key)] = value
        cooKeys[str(key)]['domain'] = url
        cooKeys[str(key)]['path'] = '/'
        cooKeys[str(key)]['expires'] = expires.strftime('%a, %d %b %Y %H:%M:%S')
        print(cooKeys)

    def get(self, key=""):
        if Version.PYVERSION_MA >= 3:
            cooKeys = cookies.SimpleCookie()

        else:
            cooKeys = Cookie.SimpleCookie()

        OsEnviron = os.environ.get("HTTP_COOKIE")
        if OsEnviron is not None:
            cooKeys.load(OsEnviron)
            if key in cooKeys:
                if cooKeys[key].value is not None:
                    return cooKeys[key].value
                else:
                    return ""
            else:
                return ""
        else:
            return ""




    def destroy(self, *args):
        if Version.PYVERSION_MA >= 3:
            cooKeys = cookies.SimpleCookie(os.environ.get('HTTP_COOKIE'))
            cook = cooKeys.items()
        else:

            cooKeys = Cookie.SimpleCookie(os.environ.get('HTTP_COOKIE'))
            cook = cooKeys.iteritems()

        if "HTTP_COOKIE" in os.environ:
            if args:

                for key in args:

                    if self.get(key) is not "":
                        return self.set(key, "", 60)
            else:
                for key, v in cook:

                    if self.get(key) is not "":
                        return self.set(key, "", 60)

        else:
            return False



# Author : BetaCodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by BetaCodings on 05/11/2019.
import os

from pytonik.Functions.url import url
class path(url):

    def __getattr__(self, item):
        return item

    def __call__(self, *args, **kwargs):

        return None

    def __init__(self, *args,  **kwargs):
        if len(args) > 0 or len(kwargs) > 0:
            if all(args) is not False:
                self.pt = self.path(*args,  **kwargs)
            else:
                self.pt = self.path(**kwargs)

        return None

    def __str__(self):

        return self.pt

    def path(self, path = "", link = False):

        DS = str('/')
        u = ""


        if path[:1] == DS or path[:1] == DS:
            DS = ""
        else:
            DS = "/"


        if os.environ.get("HTTP_HOST", '') == "":
            u = "http://localhost" + DS + os.path.basename(os.getcwd())

        if bool(link) is True:
            if os.environ.get("HTTP_HOST", '') == "":
                u = "http://localhost" + DS + os.path.basename(os.getcwd())
            else:
                u = self.url()

        return u + DS + path









from setuptools import setup, find_namespace_packages

from pytonik import Version

with open("README.md", "r") as fd:
    longdescription = fd.read()



setup(
    name='pytonik',
    version = Version.VERSION_TEXT+Version.EDITION,
    description='Pytonik is a python framework built to enhance web development fast and easy, also help web developers to build more apps with less codes',
    url="https://pytonik.readthedocs.io",
    author='pytonik',
    author_email='info@pytonik.com',
    maintainer= 'Emmanuel Essien',
    maintainer_email='emmamartinscm@gmail.com',
    packages=find_namespace_packages(include=['*', '']),
    long_description = longdescription,
    long_description_content_type='text/markdown',
    license= Version.LICENSE,
    keywords = Version.KEYWORDS,
    entry_points = {
        'console_scripts' : ['pytonik-start = pytonik.cmd.start:main', 'pytonik-server = pytonik.cmd.server:main']
                    },
    classifiers=[
        'Environment :: Console',
        'License :: OSI Approved :: MIT License',
        'Environment :: Web Environment',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Python Software Foundation License',
        'Programming Language :: Python',
        'Topic :: Office/Business',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python'
    ],
    python_requires='>=2.7, >=3.*',
)



###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###

import hashlib, os
class Hash:

    def __init__(self):
        self.hash = hashlib
        self.hashS = ""
        self.salt = os.urandom(32)
        self.storage = ""


    def md5(self, string = "",  ency = 'utf-8'):
        if string != "":
            self.hashS =  self.hash.md5(string.encode(ency)).hexdigest()
            return self.hashS

    def sha1(self, string = "",  ency = 'utf-8'):
        if string != "":
            self.hashS = self.hash.sha1(string.encode(ency)).hexdigest()
        return self.hashS

    def sha384(self, string="", ency='utf-8'):
        if string != "":
            self.hashS = self.hash.sha384(string.encode(ency)).hexdigest()
        return self.hashS

    def hex_hash(self, string = "",  hashType ='sha256',  salt = "", ency = 'utf-8'):
        if salt == "":
            vsalt = self.salt
        else:
            self.salt = salt
            vsalt = self.salt

        self.hashS = self.hash.pbkdf2_hmac(hashType, string.encode(ency),  vsalt, 100000)


        return self.hashS

    def get_hash_sha256(self, type = 'key'):

        self.storage = self.salt + self.hashS
        value = ""
        if type == "key":
            value = self.storage[:32]
        elif type == "salt":
            value = self.storage[32:]
        else:
            return "Invalid type, choose key or salt"
        return value

    def verify_hash(self, key):

        if self.hashS == key:
            return True
        else:
            return False



###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###



import importlib, os, sys, re
from ..Editor import HTMLeditor
from .. import Log
log_msg = Log.Log()

class Helpers():

    def __init__(self):
        return None

    def __call__(self, *args, **kwargs):
        return None

if "includeFile" not in dir(os):
    def includeFile(path, datag={}, datal={}):

        host = os.path.dirname(os.getcwd())
        DS = str("/")
        path = '.'+path

        split = path.split('.')

        fileExists = []
        for x in split:

            #fileExists = str(x)
            fileExists.append(x)
        try:
            dirF =  list(filter(None, fileExists))
            template_dir = host + DS + 'views' + DS + dirF[0] + DS
            engine  = dirF[1]


            if os.path.isdir(template_dir) == True:
                dataG = datag
                dataL = datal
                html_file_path = os.path.join(template_dir, "%s.html" % engine)

                with open(html_file_path) as html_file:
                    html = html_file.read()
                return HTMLeditor.Template(html).render(**dataG)

            else:
              return "The file {filepath} does not exists.".format(filepath=template_dir)

        except Exception as err:
                log_msg.error(err)
                return  err

if "callfunc" not in dir(os):
    def callfunc(myfile, myfunc, *args):
        pathname, filename = os.path.split(myfile)
        sys.path.append(os.path.abspath(pathname))
        modname = os.path.splitext(filename)[0]
        mymod = importlib.import_module(modname)
        try:
            result = getattr(mymod, myfunc)(*args)
            return result
        except Exception as err:
                log_msg.error(err)
                return  err


if "url" not in dir(os):
    def url(path = ""):
        http = os.environ.get("HTTPS")
        if http == 'on':
            url = str("https://") + os.environ.get("HTTP_HOST")
        else:
            url = str("http://") + os.environ.get("HTTP_HOST")

        return url+path

if "public_path" not in dir(os):
    def public_path(path = ""):
        DS = str('/');
        return url(DS+'public'+DS+path);


if "covert_list_dict" not in dir(os):
    def covert_list_dict(list):
        itlist = iter(list)
        try:
            return dict(zip(itlist, itlist))
        except Exception as err:
            log_msg.error(err)
            return  err



if "mvc_dir" not in dir(os):
    def mvc_dir(path):
        dpp =  path[len(str(path))-1]
        if '/' in dpp :
            newpath = path[:-1]
        else:
            newpath = path
        host = str(os.path.dirname(os.getcwd()))
        DS = str("/")
        dir = host + DS + newpath + DS
        return dir


if "key_tag" not in dir(os):
    def key_tag(tag, url="/", css=""):

        pattern = re.compile("\s*,\s*|\s+$")

        split_tag = [x for x in pattern.split(tag) if x]
        for keyword in split_tag:
            return '<li><a href="{url}" class="{css}">{keyword}</a></li>'.format(css = css, keyword = str(keyword))



###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###


import sys

VERSION = (1, 9, 7, 'beta', 2)

if VERSION[3] and VERSION[4]:
	VERSION_TEXT = '{0}.{1}.{2}{3}{4}'.format(*VERSION)
else:
	VERSION_TEXT = '{0}.{1}.{2}'.format(*VERSION[0:3])

VERSION_EXTRA = ''
LICENSE = 'MIT' #MIT
EDITION = ''  # Added in package names, after the version
KEYWORDS = "mvc, oop, module, python, framework"

PYVERSION_MA = sys.version_info.major
PYVERSION_MI = sys.version_info.minor



###
# Author : Betacodings
# Author : info@betacodings.com
# Maintainer By: Emmanuel Martins
# Maintainer Email: emmamartinscm@gmail.com
# Created by Betacodings on 2019.
###


import logging, os, datetime

class Log():
    def __init__(self):
        if os.path.isdir(os.getcwd() + '/public'):
            host = os.getcwd()  # os.path.dirname(os.getcwd())

        else:
            host = os.path.dirname(os.getcwd())

        self.locate = host
        self.logs = logging
        self.__file()
        dt = datetime

    def debug(self, string=""):
        return self.logs.debug(string)

    def info(self, string=""):

        return self.logs.info(string)

    def warning(self, string=""):

        return self.logs.warning(string)

    def error(self, string=""):

        return self.logs.error(string)

    def critical(self, string=""):

        return self.logs.critical(string)

    def __file(self):
        error_log = self.locate + "/" + 'app.log'
        if os.path.isfile(error_log) == False:
            try:
                f = open(error_log, 'a+')
                f.write("")
                f.close()
            except Exception as err:
                print(err)

        else:
            return self.logs.basicConfig(filename=error_log, level=logging.DEBUG, format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',  datefmt='%m/%d/%Y %I:%M:%S %p')
